@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkEvent.c,v 1.17.2.1 2003/07/19 01:03:25 hobbs Exp $
+ * RCS: @(#) $Id: tkEvent.c,v 1.17.2.6 2004/10/26 16:39:01 rmax Exp $
  */
 
 #include "tkPort.h"
@@ -838,7 +838,12 @@ Tk_HandleEvent(eventPtr)
 	 * available on the Windows version of Tk.
 	 */
     
+#ifdef MAC_OSX_TK
+        /* MouseWheel events are not focus specific on Mac OS X */
+	if (mask & (KeyPressMask|KeyReleaseMask)) {
+#else
 	if (mask & (KeyPressMask|KeyReleaseMask|MouseWheelMask)) {
+#endif
 	    winPtr->dispPtr->lastEventTime = eventPtr->xkey.time;
 	    winPtr = TkFocusKeyEvent(winPtr, eventPtr);
 	    if (winPtr == NULL) {
@@ -877,6 +882,7 @@ Tk_HandleEvent(eventPtr)
      */
     dispPtr = winPtr->dispPtr;
     if ((dispPtr->flags & TK_DISPLAY_USE_IM)) {
+	long im_event_mask = 0L;
 	if (!(winPtr->flags & (TK_CHECKED_IC|TK_ALREADY_DEAD))) {
 	    winPtr->flags |= TK_CHECKED_IC;
 	    if (dispPtr->inputMethod != NULL) {
@@ -932,6 +938,16 @@ Tk_HandleEvent(eventPtr)
 #endif
 	    }
 	}
+	if (winPtr->inputContext != NULL &&
+	    (eventPtr->xany.type == FocusIn)) {
+	    XGetICValues(winPtr->inputContext,
+			 XNFilterEvents, &im_event_mask, NULL);
+	    if (im_event_mask != 0L) {
+		XSelectInput(winPtr->display, winPtr->window,
+			     winPtr->atts.event_mask | im_event_mask);
+		XSetICFocus(winPtr->inputContext);
+	    }
+	}
 	if (XFilterEvent(eventPtr, None)) {
 	    goto done;
 	}
@@ -1358,6 +1374,10 @@ TkQueueEventForAllChildren(winPtr, eventPtr)
 {
     TkWindow *childPtr;
 
+    if (!Tk_IsMapped(winPtr)) {
+        return;
+    }
+    
     eventPtr->xany.window = winPtr->window;
     Tk_QueueWindowEvent(eventPtr, TCL_QUEUE_TAIL);
     
